@@ -9,7 +9,7 @@
 #
 # Copyright 2014-2016 VeriSign, Inc.
 #
-# Copyright 2016-2021 Casey Deccio
+# Copyright 2016-2024 Casey Deccio
 #
 # DNSViz is free software; you can redistribute it and/or modify
 # it under the terms of the GNU General Public License as published by
@@ -179,8 +179,10 @@ DS_DIGEST_ALGS_NOT_RECOMMENDED = ()
 DS_DIGEST_ALGS_PROHIBITED = (0, 1, 3)
 DS_DIGEST_ALGS_VALIDATION_PROHIBITED = ()
 
+RDTYPES_TTL_EXEMPT = (dns.rdatatype.NSEC3PARAM,)
+
 class RRSIGStatus(object):
-    def __init__(self, rrset, rrsig, dnskey, zone_name, reference_ts, supported_algs):
+    def __init__(self, rrset, rrsig, dnskey, zone_name, reference_ts, supported_algs, ignore_rfc8624):
         self.rrset = rrset
         self.rrsig = rrsig
         self.dnskey = dnskey
@@ -228,18 +230,32 @@ class RRSIGStatus(object):
         #
         # Signing is prohibited
         if self.rrsig.algorithm in DNSKEY_ALGS_VALIDATION_PROHIBITED:
-            self.warnings.append(Errors.AlgorithmValidationProhibited(algorithm=self.rrsig.algorithm))
+            if not ignore_rfc8624:
+                self.warnings.append(Errors.AlgorithmValidationProhibited(algorithm=self.rrsig.algorithm))
         # Validation is prohibited or, at least, not recommended
         if self.rrsig.algorithm in DNSKEY_ALGS_PROHIBITED:
-            self.warnings.append(Errors.AlgorithmProhibited(algorithm=self.rrsig.algorithm))
+            if not ignore_rfc8624:
+                self.errors.append(Errors.AlgorithmProhibited(algorithm=self.rrsig.algorithm))
         elif self.rrsig.algorithm in DNSKEY_ALGS_NOT_RECOMMENDED:
-            self.warnings.append(Errors.AlgorithmNotRecommended(algorithm=self.rrsig.algorithm))
+            if not ignore_rfc8624:
+                self.warnings.append(Errors.AlgorithmNotRecommended(algorithm=self.rrsig.algorithm))
 
-        if self.rrset.ttl_cmp:
-            if self.rrset.rrset.ttl != self.rrset.rrsig_info[self.rrsig].ttl:
-                self.warnings.append(Errors.RRsetTTLMismatch(rrset_ttl=self.rrset.rrset.ttl, rrsig_ttl=self.rrset.rrsig_info[self.rrsig].ttl))
-        if self.rrset.rrsig_info[self.rrsig].ttl > self.rrsig.original_ttl:
-            self.errors.append(Errors.OriginalTTLExceeded(rrset_ttl=self.rrset.rrset.ttl, original_ttl=self.rrsig.original_ttl))
+        if self.rrset.rrset.rdtype not in RDTYPES_TTL_EXEMPT:
+            # If we are comparing TTLs (i.e., for authoritative server responses),
+            # then check that the TTL of the RRset matches the TTL of the RRSIG
+            if self.rrset.ttl_cmp:
+                if self.rrset.rrset.ttl != self.rrset.rrsig_info[self.rrsig].ttl:
+                    self.warnings.append(Errors.RRsetTTLMismatch(rrset_ttl=self.rrset.rrset.ttl, rrsig_ttl=self.rrset.rrsig_info[self.rrsig].ttl))
+
+            # Check that the TTL of the RRset does not exceed the value in the
+            # original TTL field of the RRSIG
+            if self.rrset.rrset.ttl > self.rrsig.original_ttl:
+                self.errors.append(Errors.OriginalTTLExceededRRset(rrset_ttl=self.rrset.rrset.ttl, original_ttl=self.rrsig.original_ttl))
+
+            # Check that the TTL of the RRSIG does not exceed the value in the
+            # original TTL field of the RRSIG
+            if self.rrset.rrsig_info[self.rrsig].ttl > self.rrsig.original_ttl:
+                self.errors.append(Errors.OriginalTTLExceededRRSIG(rrsig_ttl=self.rrset.rrsig_info[self.rrsig].ttl, original_ttl=self.rrsig.original_ttl))
 
         min_ttl = min(self.rrset.rrset.ttl, self.rrset.rrsig_info[self.rrsig].ttl, self.rrsig.original_ttl)
 
@@ -253,6 +269,11 @@ class RRSIGStatus(object):
                 zn = zone_name
             self.errors.append(Errors.SignerNotZone(zone_name=fmt.humanize_name(zn), signer_name=fmt.humanize_name(self.rrsig.signer)))
 
+        if self.rrsig.labels > len(self.rrset.rrset.name) - 1:
+            self.errors.append(Errors.RRSIGLabelsExceedRRsetOwnerLabels(rrsig_labels=self.rrsig.labels, rrset_owner_labels=len(self.rrset.rrset.name) - 1))
+            if self.validation_status == RRSIG_STATUS_VALID:
+                self.validation_status = RRSIG_STATUS_INVALID
+
         if self.dnskey is not None and \
                 self.dnskey.rdata.flags & fmt.DNSKEY_FLAGS['revoke'] and self.rrsig.covers() != dns.rdatatype.DNSKEY:
             if self.rrsig.key_tag != self.dnskey.key_tag:
@@ -279,7 +300,7 @@ class RRSIGStatus(object):
             if self.validation_status == RRSIG_STATUS_VALID:
                 self.validation_status = RRSIG_STATUS_EXPIRED
             self.errors.append(Errors.ExpirationInPast(expiration=fmt.timestamp_to_datetime(self.rrsig.expiration), reference_time=fmt.timestamp_to_datetime(self.reference_ts)))
-        elif self.reference_ts + min_ttl >= self.rrsig.expiration:
+        elif self.reference_ts + min_ttl > self.rrsig.expiration:
             self.errors.append(Errors.TTLBeyondExpiration(expiration=fmt.timestamp_to_datetime(self.rrsig.expiration), rrsig_ttl=min_ttl, reference_time=fmt.timestamp_to_datetime(self.reference_ts)))
         elif self.reference_ts + CLOCK_SKEW_WARNING >= self.rrsig.expiration:
             self.warnings.append(Errors.ExpirationWithinClockSkew(expiration=fmt.timestamp_to_datetime(self.rrsig.expiration), reference_time=fmt.timestamp_to_datetime(self.reference_ts)))
@@ -348,7 +369,10 @@ class RRSIGStatus(object):
             d['servers'] = servers
 
             if map_ip_to_ns_name is not None:
-                ns_names = list(set([lb2s(map_ip_to_ns_name(s)[0][0].canonicalize().to_text()) for s in servers]))
+                try:
+                    ns_names = list(set([lb2s(map_ip_to_ns_name(s)[0][0].canonicalize().to_text()) for s in servers]))
+                except IndexError:
+                    ns_names = []
                 ns_names.sort()
                 d['ns_names'] = ns_names
 
@@ -378,7 +402,7 @@ class RRSIGStatus(object):
         return d
 
 class DSStatus(object):
-    def __init__(self, ds, ds_meta, dnskey, supported_digest_algs):
+    def __init__(self, ds, ds_meta, dnskey, supported_digest_algs, ignore_rfc8624):
         self.ds = ds
         self.ds_meta = ds_meta
         self.dnskey = dnskey
@@ -423,12 +447,15 @@ class DSStatus(object):
         #
         # Signing is prohibited
         if self.ds.digest_type in DS_DIGEST_ALGS_VALIDATION_PROHIBITED:
-            self.warnings.append(Errors.DigestAlgorithmValidationProhibited(algorithm=self.ds.digest_type))
+            if not ignore_rfc8624:
+                self.warnings.append(Errors.DigestAlgorithmValidationProhibited(algorithm=self.ds.digest_type))
         # Validation is prohibited or, at least, not recommended
         if self.ds.digest_type in DS_DIGEST_ALGS_PROHIBITED:
-            self.warnings.append(Errors.DigestAlgorithmProhibited(algorithm=self.ds.digest_type))
+            if not ignore_rfc8624:
+                self.errors.append(Errors.DigestAlgorithmProhibited(algorithm=self.ds.digest_type))
         elif self.ds.digest_type in DS_DIGEST_ALGS_NOT_RECOMMENDED:
-            self.warnings.append(Errors.DigestAlgorithmNotRecommended(algorithm=self.ds.digest_type))
+            if not ignore_rfc8624:
+                self.warnings.append(Errors.DigestAlgorithmNotRecommended(algorithm=self.ds.digest_type))
 
         if self.dnskey is not None and \
                 self.dnskey.rdata.flags & fmt.DNSKEY_FLAGS['revoke']:
@@ -521,7 +548,10 @@ class DSStatus(object):
             d['servers'] = servers
 
             if map_ip_to_ns_name is not None:
-                ns_names = list(set([lb2s(map_ip_to_ns_name(s)[0][0].canonicalize().to_text()) for s in servers]))
+                try:
+                    ns_names = list(set([lb2s(map_ip_to_ns_name(s)[0][0].canonicalize().to_text()) for s in servers]))
+                except IndexError:
+                    ns_names = []
                 ns_names.sort()
                 d['ns_names'] = ns_names
 
@@ -699,7 +729,10 @@ class NSECStatusNXDOMAIN(NSECStatus):
             d['servers'] = servers
 
             if map_ip_to_ns_name is not None:
-                ns_names = list(set([lb2s(map_ip_to_ns_name(s)[0][0].canonicalize().to_text()) for s in servers]))
+                try:
+                    ns_names = list(set([lb2s(map_ip_to_ns_name(s)[0][0].canonicalize().to_text()) for s in servers]))
+                except IndexError:
+                    ns_names = []
                 ns_names.sort()
                 d['ns_names'] = ns_names
 
@@ -831,8 +864,14 @@ class NSECStatusNODATA(NSECStatus):
             try:
                 self.nsec_for_wildcard_name = nsec_set_info.rrsets[self.wildcard_name]
                 self.wildcard_has_rdtype = nsec_set_info.rdtype_exists_in_bitmap(self.wildcard_name, self.rdtype)
+                self.wildcard_has_ns = nsec_set_info.rdtype_exists_in_bitmap(self.wildcard_name, dns.rdatatype.NS)
+                self.wildcard_has_ds = nsec_set_info.rdtype_exists_in_bitmap(self.wildcard_name, dns.rdatatype.DS)
+                self.wildcard_has_soa = nsec_set_info.rdtype_exists_in_bitmap(self.wildcard_name, dns.rdatatype.SOA)
             except KeyError:
-                pass
+                self.wildcard_has_rdtype = False
+                self.wildcard_has_ns = False
+                self.wildcard_has_ds = False
+                self.wildcard_has_soa = False
 
         # check for covering of the origin
         self.nsec_names_covering_origin = {}
@@ -876,6 +915,20 @@ class NSECStatusNODATA(NSECStatus):
                 self.errors.append(Errors.SnameCoveredNODATANSEC(sname=fmt.humanize_name(self.qname)))
                 self.validation_status = NSEC_STATUS_INVALID
         elif self.nsec_for_wildcard_name: # implies wildcard_name, which implies nsec_names_covering_qname
+            if self.rdtype == dns.rdatatype.DS or self.referral:
+                if self.is_zone and not self.wildcard_has_ns:
+                    self.errors.append(Errors.ReferralWithoutNSBitNSEC(sname=fmt.humanize_name(self.wildcard_name)))
+                    self.validation_status = NSEC_STATUS_INVALID
+                if self.wildcard_has_ds:
+                    self.errors.append(Errors.ReferralWithDSBitNSEC(sname=fmt.humanize_name(self.wildcard_name)))
+                    self.validation_status = NSEC_STATUS_INVALID
+                if self.wildcard_has_soa:
+                    self.errors.append(Errors.ReferralWithSOABitNSEC(sname=fmt.humanize_name(self.wildcard_name)))
+                    self.validation_status = NSEC_STATUS_INVALID
+            else:
+                if self.has_rdtype:
+                    self.errors.append(Errors.StypeInBitmapNODATANSEC(sname=fmt.humanize_name(self.qname), stype=dns.rdatatype.to_text(self.rdtype)))
+                    self.validation_status = NSEC_STATUS_INVALID
             if self.wildcard_has_rdtype:
                 self.validation_status = NSEC_STATUS_INVALID
                 self.errors.append(Errors.StypeInBitmapNODATANSEC(sname=fmt.humanize_name(self.wildcard_name), stype=dns.rdatatype.to_text(self.rdtype)))
@@ -964,7 +1017,10 @@ class NSECStatusNODATA(NSECStatus):
             d['servers'] = servers
 
             if map_ip_to_ns_name is not None:
-                ns_names = list(set([lb2s(map_ip_to_ns_name(s)[0][0].canonicalize().to_text()) for s in servers]))
+                try:
+                    ns_names = list(set([lb2s(map_ip_to_ns_name(s)[0][0].canonicalize().to_text()) for s in servers]))
+                except IndexError:
+                    ns_names = []
                 ns_names.sort()
                 d['ns_names'] = ns_names
 
@@ -1016,7 +1072,7 @@ class NSEC3Status(object):
         return None
 
 class NSEC3StatusNXDOMAIN(NSEC3Status):
-    def __init__(self, qname, rdtype, origin, is_zone, nsec_set_info):
+    def __init__(self, qname, rdtype, origin, is_zone, nsec_set_info, ignore_rfc9276):
         self.qname = qname
         self.origin = origin
         self.is_zone = is_zone
@@ -1030,8 +1086,14 @@ class NSEC3StatusNXDOMAIN(NSEC3Status):
         self.nsec_names_covering_qname = {}
         self.nsec_names_covering_wildcard = {}
         self.opt_out = None
+        self.iterations = 0
+        self.salt = None
 
         for (salt, alg, iterations), nsec3_names in nsec_set_info.nsec3_params.items():
+            if iterations > self.iterations:
+                self.iterations = iterations
+            if salt:
+                self.salt = salt
             digest_name = nsec_set_info.get_digest_name_for_nsec3(self.qname, self.origin, salt, alg, iterations)
             if self.qname not in self.name_digest_map:
                 self.name_digest_map[self.qname] = {}
@@ -1070,7 +1132,7 @@ class NSEC3StatusNXDOMAIN(NSEC3Status):
                     self.name_digest_map[wildcard_name] = {}
                 self.name_digest_map[wildcard_name][(salt, alg, iterations)] = digest_name
 
-        self._set_validation_status(nsec_set_info)
+        self._set_validation_status(nsec_set_info, ignore_rfc9276)
 
     def __str__(self):
         return 'NSEC3 record(s) proving the non-existence (NXDOMAIN) of %s' % (fmt.humanize_name(self.qname))
@@ -1085,13 +1147,18 @@ class NSEC3StatusNXDOMAIN(NSEC3Status):
     def _set_closest_encloser(self, nsec_set_info):
         self.closest_encloser = nsec_set_info.get_closest_encloser(self.qname, self.origin)
 
-    def _set_validation_status(self, nsec_set_info):
+    def _set_validation_status(self, nsec_set_info, ignore_rfc9276):
         self.validation_status = NSEC_STATUS_VALID
         valid_algs, invalid_algs = nsec_set_info.get_algorithm_support()
         if invalid_algs:
             invalid_alg_err = Errors.UnsupportedNSEC3Algorithm(algorithm=list(invalid_algs)[0])
         else:
             invalid_alg_err = None
+        if not ignore_rfc9276:
+            if self.iterations > 0:
+                self.errors.append(Errors.NonZeroNSEC3IterationCount())
+            if self.salt is not None:
+                self.warnings.append(Errors.NonEmptyNSEC3Salt())
         if not self.closest_encloser:
             self.validation_status = NSEC_STATUS_INVALID
             if valid_algs:
@@ -1228,7 +1295,10 @@ class NSEC3StatusNXDOMAIN(NSEC3Status):
             d['servers'] = servers
 
             if map_ip_to_ns_name is not None:
-                ns_names = list(set([lb2s(map_ip_to_ns_name(s)[0][0].canonicalize().to_text()) for s in servers]))
+                try:
+                    ns_names = list(set([lb2s(map_ip_to_ns_name(s)[0][0].canonicalize().to_text()) for s in servers]))
+                except IndexError:
+                    ns_names = []
                 ns_names.sort()
                 d['ns_names'] = ns_names
 
@@ -1258,9 +1328,9 @@ class NSEC3StatusNXDOMAIN(NSEC3Status):
         return d
 
 class NSEC3StatusWildcard(NSEC3StatusNXDOMAIN):
-    def __init__(self, qname, wildcard_name, rdtype, origin, is_zone, nsec_set_info):
+    def __init__(self, qname, wildcard_name, rdtype, origin, is_zone, nsec_set_info, ignore_rfc9276):
         self.wildcard_name = wildcard_name
-        super(NSEC3StatusWildcard, self).__init__(qname, rdtype, origin, is_zone, nsec_set_info)
+        super(NSEC3StatusWildcard, self).__init__(qname, rdtype, origin, is_zone, nsec_set_info, ignore_rfc9276)
 
     def _set_closest_encloser(self, nsec_set_info):
         super(NSEC3StatusWildcard, self)._set_closest_encloser(nsec_set_info)
@@ -1277,8 +1347,13 @@ class NSEC3StatusWildcard(NSEC3StatusNXDOMAIN):
     def __hash__(self):
         return hash(id(self))
 
-    def _set_validation_status(self, nsec_set_info):
+    def _set_validation_status(self, nsec_set_info, ignore_rfc9276):
         self.validation_status = NSEC_STATUS_VALID
+        if not ignore_rfc9276:
+            if self.iterations > 0:
+                self.errors.append(Errors.NonZeroNSEC3IterationCount())
+            if self.salt is not None:
+                self.warnings.append(Errors.NonEmptyNSEC3Salt())
         if not self.nsec_names_covering_qname:
             self.validation_status = NSEC_STATUS_INVALID
             valid_algs, invalid_algs = nsec_set_info.get_algorithm_support()
@@ -1328,7 +1403,7 @@ class NSEC3StatusWildcard(NSEC3StatusNXDOMAIN):
         return d
 
 class NSEC3StatusNODATA(NSEC3Status):
-    def __init__(self, qname, rdtype, origin, is_zone, nsec_set_info):
+    def __init__(self, qname, rdtype, origin, is_zone, nsec_set_info, ignore_rfc9276):
         self.qname = qname
         self.rdtype = rdtype
         self.origin = origin
@@ -1354,8 +1429,14 @@ class NSEC3StatusNODATA(NSEC3Status):
         self.opt_out = None
         self.wildcard_has_rdtype = False
         self.wildcard_has_cname = False
+        self.iterations = 0
+        self.salt = None
 
         for (salt, alg, iterations), nsec3_names in nsec_set_info.nsec3_params.items():
+            if iterations > self.iterations:
+                self.iterations = iterations
+            if salt:
+                self.salt = salt
             digest_name = nsec_set_info.get_digest_name_for_nsec3(self.qname, self.origin, salt, alg, iterations)
             if self.qname not in self.name_digest_map:
                 self.name_digest_map[self.qname] = {}
@@ -1400,7 +1481,7 @@ class NSEC3StatusNODATA(NSEC3Status):
                                 if nsec_set_info.rrsets[nsec_name].rrset[0].flags & 0x01:
                                     self.opt_out = True
 
-        self._set_validation_status(nsec_set_info)
+        self._set_validation_status(nsec_set_info, ignore_rfc9276)
 
     def __str__(self):
         return 'NSEC3 record(s) proving non-existence (NODATA) of %s/%s' % (fmt.humanize_name(self.qname), dns.rdatatype.to_text(self.rdtype))
@@ -1412,13 +1493,18 @@ class NSEC3StatusNODATA(NSEC3Status):
     def __hash__(self):
         return hash(id(self))
 
-    def _set_validation_status(self, nsec_set_info):
+    def _set_validation_status(self, nsec_set_info, ignore_rfc9276):
         self.validation_status = NSEC_STATUS_VALID
         valid_algs, invalid_algs = nsec_set_info.get_algorithm_support()
         if invalid_algs:
             invalid_alg_err = Errors.UnsupportedNSEC3Algorithm(algorithm=list(invalid_algs)[0])
         else:
             invalid_alg_err = None
+        if not ignore_rfc9276:
+            if self.iterations > 0:
+                self.errors.append(Errors.NonZeroNSEC3IterationCount())
+            if self.salt is not None:
+                self.warnings.append(Errors.NonEmptyNSEC3Salt())
         if self.nsec_for_qname:
             # RFC 4035 5.2, 6840 4.4
             if self.rdtype == dns.rdatatype.DS or self.referral:
@@ -1592,7 +1678,10 @@ class NSEC3StatusNODATA(NSEC3Status):
             d['servers'] = servers
 
             if map_ip_to_ns_name is not None:
-                ns_names = list(set([lb2s(map_ip_to_ns_name(s)[0][0].canonicalize().to_text()) for s in servers]))
+                try:
+                    ns_names = list(set([lb2s(map_ip_to_ns_name(s)[0][0].canonicalize().to_text()) for s in servers]))
+                except IndexError:
+                    ns_names = []
                 ns_names.sort()
                 d['ns_names'] = ns_names
 
@@ -1692,7 +1781,10 @@ class CNAMEFromDNAMEStatus(object):
             d['servers'] = servers
 
             if map_ip_to_ns_name is not None:
-                ns_names = list(set([lb2s(map_ip_to_ns_name(s)[0][0].canonicalize().to_text()) for s in servers]))
+                try:
+                    ns_names = list(set([lb2s(map_ip_to_ns_name(s)[0][0].canonicalize().to_text()) for s in servers]))
+                except IndexError:
+                    ns_names = []
                 ns_names.sort()
                 d['ns_names'] = ns_names
 
