@@ -6,7 +6,7 @@
 #
 # Copyright 2014-2016 VeriSign, Inc.
 #
-# Copyright 2016-2021 Casey Deccio
+# Copyright 2016-2024 Casey Deccio
 #
 # DNSViz is free software; you can redistribute it and/or modify
 # it under the terms of the GNU General Public License as published by
@@ -42,9 +42,10 @@ except ImportError:
 import dns.exception, dns.name
 
 from dnsviz.analysis import OfflineDomainNameAnalysis, DNS_RAW_VERSION
+from dnsviz.analysis.errors import ExistingCovered
 from dnsviz.config import DNSVIZ_SHARE_PATH, JQUERY_PATH, JQUERY_UI_PATH, JQUERY_UI_CSS_PATH, RAPHAEL_PATH
 from dnsviz.format import latin1_binary_to_string as lb2s
-from dnsviz.util import get_trusted_keys, get_default_trusted_keys
+from dnsviz.util import get_trusted_keys, get_default_trusted_keys, io_try_buffered
 
 # If the import of DNSAuthGraph fails because of the lack of pygraphviz, it
 # will be reported later
@@ -58,6 +59,7 @@ except ImportError:
     else:
         raise
 
+IGNORE_UNLESS_ERRORS = set([dns.rdatatype.HINFO, dns.rdatatype.NSEC3PARAM])
 LOCAL_MEDIA_URL = 'file://' + DNSVIZ_SHARE_PATH
 DNSSEC_TEMPLATE_FILE = os.path.join(DNSVIZ_SHARE_PATH, 'html', 'dnssec-template.html')
 
@@ -154,8 +156,8 @@ class GraphArgHelper:
         self.parser = argparse.ArgumentParser(description='Graph the assessment of diagnostic DNS queries', prog=prog)
 
         # python3/python2 dual compatibility
-        stdin_buffer = io.open(sys.stdin.fileno(), 'rb', closefd=False)
-        stdout_buffer = io.open(sys.stdout.fileno(), 'wb', closefd=False)
+        stdin_buffer = io_try_buffered(sys.stdin, 'rb', closefd=False)
+        stdout_buffer = io_try_buffered(sys.stdout, 'wb', closefd=False)
 
         try:
             self.parser.add_argument('-f', '--names-file',
@@ -205,10 +207,14 @@ class GraphArgHelper:
                 type=self.comma_separated_ints_set,
                 action='store', metavar='<digest_alg>,[<digest_alg>...]',
                 help='Support only the specified DNSSEC digest algorithm(s)')
-        self.parser.add_argument('-b', '--validate-prohibited-algs',
+        self.parser.add_argument('--ignore-rfc8624',
                 const=True, default=False,
                 action='store_const',
-                help='Validate algorithms for which validation is otherwise prohibited')
+                help='Ignore errors associated with RFC 8624, DNSSEC algorithm implementation requirements')
+        self.parser.add_argument('--ignore-rfc9276',
+                const=True, default=False,
+                action='store_const',
+                help='Ignore errors associated with RFC 9276, NSEC3 parameter settings')
         self.parser.add_argument('-C', '--enforce-cookies',
                 const=True, default=False,
                 action='store_const',
@@ -217,6 +223,14 @@ class GraphArgHelper:
                 const=True, default=False,
                 action='store_const',
                 help='Allow private IP addresses for authoritative DNS servers')
+        self.parser.add_argument('--trust-cdnskey-cds',
+                const=True, default=False,
+                action='store_const',
+                help='Trust all CDNSKEY/CDS records')
+        self.parser.add_argument('--multi-signer',
+                const=True, default=False,
+                action='store_const',
+                help='Don\'t issue errors for missing KSKs with DS RRs (e.g., for multi-signer setups)')
         self.parser.add_argument('-R', '--rr-types',
                 type=self.comma_separated_dns_types,
                 action='store', metavar='<type>,[<type>...]',
@@ -468,13 +482,21 @@ def main(argv):
 
         G = DNSAuthGraph()
         for name_obj in name_objs:
-            name_obj.populate_status(arghelper.trusted_keys, supported_algs=arghelper.args.algorithms, supported_digest_algs=arghelper.args.digest_algorithms, validate_prohibited_algs=arghelper.args.validate_prohibited_algs)
+            name_obj.populate_status(arghelper.trusted_keys, supported_algs=arghelper.args.algorithms, supported_digest_algs=arghelper.args.digest_algorithms, ignore_rfc8624=arghelper.args.ignore_rfc8624, ignore_rfc9276=arghelper.args.ignore_rfc9276, trust_cdnskey_cds=arghelper.args.trust_cdnskey_cds, multi_signer=arghelper.args.multi_signer)
+            has_warnings, has_errors = name_obj.queries_with_errors_warnings()
+            has_warnings_or_errors = has_warnings.union(has_errors)
             for qname, rdtype in name_obj.queries:
                 if arghelper.args.rr_types is None:
                     # if rdtypes was not specified, then graph all, with some
                     # exceptions
                     if name_obj.is_zone() and rdtype in (dns.rdatatype.DNSKEY, dns.rdatatype.DS, dns.rdatatype.DLV):
                         continue
+
+                    # Ignore some data types unless they are explicitly
+                    # requested or there are warnings or errors associated with
+                    # them.
+                    if rdtype in IGNORE_UNLESS_ERRORS and (qname, rdtype) not in has_warnings_or_errors:
+                        continue
                 else:
                     # if rdtypes was specified, then only graph rdtypes that
                     # were specified
@@ -482,6 +504,13 @@ def main(argv):
                         continue
                 G.graph_rrset_auth(name_obj, qname, rdtype)
 
+            # Also graph the NODATA query for the zone name, if applicable
+            if not name_obj.is_zone() and name_obj.zone is not None:
+                z_has_warnings, z_has_errors = name_obj.zone.queries_with_errors_warnings(ExistingCovered)
+                z_has_warnings_or_errors = z_has_warnings.union(z_has_errors)
+                if (name_obj.zone.nxrrset_name, name_obj.zone.nxrrset_rdtype) in z_has_warnings_or_errors:
+                    G.graph_rrset_auth(name_obj.zone, name_obj.zone.nxrrset_name, name_obj.zone.nxrrset_rdtype)
+
             if arghelper.args.rr_types is not None:
                 for rdtype in arghelper.args.rr_types:
                     if (name_obj.name, rdtype) not in name_obj.queries:
