@@ -6,7 +6,7 @@
 #
 # Copyright 2014-2016 VeriSign, Inc.
 #
-# Copyright 2016-2021 Casey Deccio
+# Copyright 2016-2024 Casey Deccio
 #
 # DNSViz is free software; you can redistribute it and/or modify
 # it under the terms of the GNU General Public License as published by
@@ -43,7 +43,7 @@ import dns.exception, dns.name
 
 from dnsviz.analysis import OfflineDomainNameAnalysis, DNS_RAW_VERSION
 from dnsviz.format import latin1_binary_to_string as lb2s
-from dnsviz.util import get_trusted_keys
+from dnsviz.util import get_trusted_keys, io_try_buffered
 
 # If the import of DNSAuthGraph fails because of the lack of pygraphviz, it
 # will be reported later
@@ -136,16 +136,23 @@ def color_json(s):
 
 def test_pygraphviz():
     try:
-        from pygraphviz import release
         try:
-            major, minor = release.version.split('.')[:2]
+            # pygraphviz < 1.7 used pygraphviz.release.version
+            from pygraphviz import release
+            version = release.version
+        except ImportError:
+            # pygraphviz 1.7 changed to pygraphviz.__version__
+            from pygraphviz import __version__
+            version = __version__
+        try:
+            major, minor = version.split('.')[:2]
             major = int(major)
             minor = int(re.sub(r'(\d+)[^\d].*', r'\1', minor))
             if (major, minor) < (1,3):
-                logger.error('''pygraphviz version >= 1.3 is required, but version %s is installed.''' % release.version)
+                logger.error('''pygraphviz version >= 1.3 is required, but version %s is installed.''' % version)
                 sys.exit(2)
         except ValueError:
-            logger.error('''pygraphviz version >= 1.3 is required, but version %s is installed.''' % release.version)
+            logger.error('''pygraphviz version >= 1.3 is required, but version %s is installed.''' % version)
             sys.exit(2)
     except ImportError:
         logger.error('''pygraphviz is required, but not installed.''')
@@ -170,8 +177,8 @@ class GrokArgHelper:
         self.parser = argparse.ArgumentParser(description='Assess diagnostic DNS queries', prog=prog)
 
         # python3/python2 dual compatibility
-        stdin_buffer = io.open(sys.stdin.fileno(), 'rb', closefd=False)
-        stdout_buffer = io.open(sys.stdout.fileno(), 'wb', closefd=False)
+        stdin_buffer = io_try_buffered(sys.stdin, 'rb', closefd=False)
+        stdout_buffer = io_try_buffered(sys.stdout, 'wb', closefd=False)
 
         try:
             self.parser.add_argument('-f', '--names-file',
@@ -221,10 +228,14 @@ class GrokArgHelper:
                 type=self.comma_separated_ints_set,
                 action='store', metavar='<digest_alg>,[<digest_alg>...]',
                 help='Support only the specified DNSSEC digest algorithm(s)')
-        self.parser.add_argument('-b', '--validate-prohibited-algs',
+        self.parser.add_argument('--ignore-rfc8624',
                 const=True, default=False,
                 action='store_const',
-                help='Validate algorithms for which validation is otherwise prohibited')
+                help='Ignore errors associated with RFC 8624, DNSSEC algorithm implementation requirements')
+        self.parser.add_argument('--ignore-rfc9276',
+                const=True, default=False,
+                action='store_const',
+                help='Ignore errors associated with RFC 9276, NSEC3 parameter settings')
         self.parser.add_argument('-C', '--enforce-cookies',
                 const=True, default=False,
                 action='store_const',
@@ -233,6 +244,14 @@ class GrokArgHelper:
                 const=True, default=False,
                 action='store_const',
                 help='Allow private IP addresses for authoritative DNS servers')
+        self.parser.add_argument('--trust-cdnskey-cds',
+                const=True, default=False,
+                action='store_const',
+                help='Trust all CDNSKEY/CDS records')
+        self.parser.add_argument('--multi-signer',
+                const=True, default=False,
+                action='store_const',
+                help='Don\'t issue errors for missing KSKs with DS RRs (e.g., for multi-signer setups)')
         self.parser.add_argument('-o', '--output-file',
                 type=argparse.FileType('wb'), default=stdout_buffer,
                 action='store', metavar='<filename>',
@@ -458,7 +477,7 @@ def main(argv):
 
         d = OrderedDict()
         for name_obj in name_objs:
-            name_obj.populate_status(arghelper.trusted_keys, supported_algs=arghelper.args.algorithms, supported_digest_algs=arghelper.args.digest_algorithms, validate_prohibited_algs=arghelper.args.validate_prohibited_algs)
+            name_obj.populate_status(arghelper.trusted_keys, supported_algs=arghelper.args.algorithms, supported_digest_algs=arghelper.args.digest_algorithms, ignore_rfc8624=arghelper.args.ignore_rfc8624, ignore_rfc9276=arghelper.args.ignore_rfc9276, trust_cdnskey_cds=arghelper.args.trust_cdnskey_cds, multi_signer=arghelper.args.multi_signer)
 
             if arghelper.trusted_keys:
                 G = DNSAuthGraph()
