@@ -6,7 +6,7 @@
 #
 # Copyright 2014-2016 VeriSign, Inc.
 #
-# Copyright 2016-2021 Casey Deccio
+# Copyright 2016-2024 Casey Deccio
 #
 # DNSViz is free software; you can redistribute it and/or modify
 # it under the terms of the GNU General Public License as published by
@@ -119,6 +119,7 @@ def _cleanup_tm():
     global tm
     if tm is not None:
         tm.close()
+        tm = None
 
 def _init_stub_resolver():
     global resolver
@@ -341,10 +342,11 @@ class ZoneFileToServe:
 options {
     directory "%(dir)s";
     pid-file "%(named_pid)s";
-    listen-on port %(port)d { localhost; };
-    listen-on-v6 port %(port)d { localhost; };
+    listen-on port %(port)d { 127.0.0.1; };
+    listen-on-v6 port %(port)d { ::1; };
     recursion no;
     notify no;
+    dnssec-validation no;
 };
 controls {};
 zone "%(zone_name)s" {
@@ -352,7 +354,7 @@ zone "%(zone_name)s" {
     file "%(zone_file)s";
 };
 logging {
-	channel info_file { file "%(named_log)s"; severity info; };
+	channel info_file { file "%(named_log)s"; severity debug; };
 	category default { info_file; };
 	category unmatched { null; };
 };
@@ -365,7 +367,7 @@ $TTL 600
 '''
     ZONEFILE_TEMPLATE_A = '@ IN A 127.0.0.1\n'
     ZONEFILE_TEMPLATE_AAAA = '@ IN AAAA ::1\n'
-    USAGE_RE = re.compile(r'usage:', re.IGNORECASE)
+    ERROR_RE = re.compile(r'permission\s+denied|error:', re.IGNORECASE)
 
     def __init__(self, domain, filename):
         self.domain = domain
@@ -434,15 +436,15 @@ $TTL 600
             raise MissingExecutablesError('The options used require %s.  Please ensure that it is installed and in PATH (%s).' % (self.NAMED_CHECKCONF, e))
 
         (stdout, stderr) = p.communicate()
-        if p.returncode != 0:
+        if False: #p.returncode != 0:
             stdout = stdout.decode('utf-8')
             self._cleanup_process()
-            raise ZoneFileServiceError('There was an problem with the zone file for "%s":\n%s' % (args['zone_name'], stdout))
+            raise ZoneFileServiceError('There was a problem with the zone file for "%s":\n%s' % (args['zone_name'], stdout))
 
-        named_cmd_without_log = [self.NAMED, '-c', args['named_conf']]
-        named_cmd_with_log = named_cmd_without_log + ['-L', args['named_log']]
-        checked_usage = False
-        for named_cmd in (named_cmd_with_log, named_cmd_without_log):
+        named_cmd_without_debug = [self.NAMED, '-c', args['named_conf']]
+        named_cmd_with_debug = named_cmd_without_debug + ['-g']
+        started = False
+        for named_cmd in (named_cmd_without_debug, named_cmd_with_debug):
             try:
                 p = subprocess.Popen(named_cmd, stdout=subprocess.PIPE, stderr=subprocess.STDOUT, env=env)
             except OSError as e:
@@ -451,14 +453,10 @@ $TTL 600
 
             (stdout, stderr) = p.communicate()
             if p.returncode == 0:
+                started = True
                 break
 
             stdout = stdout.decode('utf-8')
-            if not checked_usage and self.USAGE_RE.search(stdout):
-                # Versions of BIND pre 9.11 don't support -L, so fall back to without -L
-                checked_usage = True
-                continue
-
             try:
                 with io.open(args['named_log'], 'r', encoding='utf-8') as fh:
                     log = fh.read()
@@ -466,8 +464,19 @@ $TTL 600
                 log = ''
             if not log:
                 log = stdout
+
+            if log:
+                break
+
+        if not started:
+            newlog = ''
+            lines = log.splitlines()
+            for line in lines:
+                if self.ERROR_RE.search(line):
+                    newlog += line + '\n'
+
             self._cleanup_process()
-            raise ZoneFileServiceError('There was an problem executing %s to serve the "%s" zone:\n%s' % (self.NAMED, args['zone_name'], log))
+            raise ZoneFileServiceError('There was an problem executing %s to serve the "%s" zone:\n%s' % (self.NAMED, args['zone_name'], newlog))
 
         try:
             with io.open(args['named_pid'], 'r', encoding='utf-8') as fh:
@@ -491,7 +500,11 @@ class NameServerMappingsForDomain(object):
     _allow_stop_at = None
     _handle_file_arg = None
 
-    def __init__(self, domain, stop_at, resolver):
+    _resolvers_initialized = False
+    _stub_resolver = None
+    _full_resolver = None
+
+    def __init__(self, domain, stop_at):
         if not (self._allow_file is not None and \
                 self._allow_name_only is not None and \
                 self._allow_addr_only is not None and \
@@ -503,7 +516,6 @@ class NameServerMappingsForDomain(object):
             raise argparse.ArgumentTypeError('The "+" may not be specified with this option')
 
         self.domain = domain
-        self._resolver = resolver
         self._nsi = 1
 
         self.delegation_mapping = {}
@@ -513,6 +525,23 @@ class NameServerMappingsForDomain(object):
 
         self.delegation_mapping[(self.domain, dns.rdatatype.NS)] = dns.rrset.RRset(self.domain, dns.rdataclass.IN, dns.rdatatype.NS)
 
+    @classmethod
+    def init_resolvers(cls):
+        if not NameServerMappingsForDomain._resolvers_initialized:
+            tm = transport.DNSQueryTransportManager()
+            try:
+                NameServerMappingsForDomain._stub_resolver = Resolver.from_file(RESOLV_CONF, StandardRecursiveQueryCD, transport_manager=tm)
+            except ResolvConfError:
+                pass
+            NameServerMappingsForDomain._full_resolver = PrivateFullResolver(transport_manager=tm)
+            NameServerMappingsForDomain._resolvers_initialized = True
+
+    @classmethod
+    def cleanup_resolvers(cls):
+        NameServerMappingsForDomain._stub_resolver = None
+        NameServerMappingsForDomain._full_resolver = None
+        NameServerMappingsForDomain._resolvers_initialized = False
+
     @classmethod
     def _strip_port(cls, s):
         # Determine whether there is a port attached to the end
@@ -540,8 +569,14 @@ class NameServerMappingsForDomain(object):
             self._handle_name_addr_mapping(name_addr)
 
     def _handle_name_no_addr(self, name, port):
+        resolver = None
+        self.init_resolvers()
+        if self._stub_resolver is not None:
+            resolver = self._stub_resolver
+        else:
+            resolver = self._full_resolver
         query_tuples = ((name, dns.rdatatype.A, dns.rdataclass.IN), (name, dns.rdatatype.AAAA, dns.rdataclass.IN))
-        answer_map = self._resolver.query_multiple_for_answer(*query_tuples)
+        answer_map = resolver.query_multiple_for_answer(*query_tuples)
         found_answer = False
         for (n, rdtype, rdclass) in answer_map:
             a = answer_map[(n, rdtype, rdclass)]
@@ -762,7 +797,7 @@ class RecursiveServersForDomain(NameServerMappingsForDomain):
     _handle_file_arg = None
 
 class DSForDomain:
-    def __init__(self, domain, stop_at, resolver):
+    def __init__(self, domain, stop_at):
         self.domain = domain
 
         if stop_at and not self._allow_stop_at:
@@ -817,9 +852,6 @@ class DSForDomain:
 class DomainListArgHelper:
     STOP_RE = re.compile(r'^(.*)\+$')
 
-    def __init__(self, resolver):
-        self._resolver = resolver
-    
     @classmethod
     def _strip_stop_marker(cls, s):
         match = cls.STOP_RE.search(s)
@@ -855,14 +887,14 @@ class DomainListArgHelper:
         if list_arg is not None:
             list_arg = list_arg.strip()
 
-        obj = cls(domain, stop_at, self._resolver)
+        obj = cls(domain, stop_at)
 
         if list_arg:
             obj.handle_list_arg(list_arg)
         return obj
 
     def _handle_list_arg(self, cls, list_arg):
-        obj = cls(WILDCARD_EXPLICIT_DELEGATION, False, self._resolver)
+        obj = cls(WILDCARD_EXPLICIT_DELEGATION, False)
         obj.handle_list_arg(list_arg)
         return obj
 
@@ -881,8 +913,7 @@ class DomainListArgHelper:
 class ArgHelper:
     BRACKETS_RE = re.compile(r'^\[(.*)\]$')
 
-    def __init__(self, resolver, logger):
-        self._resolver = resolver
+    def __init__(self, logger):
         self.parser = None
 
         self.odd_ports = {}
@@ -906,13 +937,12 @@ class ArgHelper:
         self.args = None
         self._arg_mapping = None
 
-        self._resolver = resolver
         self._logger = logger
         self._zones_to_serve = []
 
     def build_parser(self, prog):
         self.parser = argparse.ArgumentParser(description='Issue diagnostic DNS queries', prog=prog)
-        helper = DomainListArgHelper(self._resolver)
+        helper = DomainListArgHelper()
 
         # python3/python2 dual compatibility
         stdout_buffer = io.open(sys.stdout.fileno(), 'wb', closefd=False)
@@ -997,10 +1027,10 @@ class ArgHelper:
                 type=helper.ds_for_domain, default=[],
                 action='append', metavar='<domain>:"<ds>"[,"<ds>"...]',
                 help='Use the specified DS records for a domain')
-        self.parser.add_argument('-n', '--nsid',
-                const=self.nsid_option(),
+        self.parser.add_argument('--no-nsid',
+                const=True, default=False,
                 action='store_const',
-                help='Use the NSID EDNS option in queries')
+                help='Do not use the NSID EDNS option in queries')
         self.parser.add_argument('-e', '--client-subnet',
                 type=self.ecs_option,
                 action='store', metavar='<subnet>[:<prefix_len>]',
@@ -1061,7 +1091,10 @@ class ArgHelper:
             if e.errno == errno.EADDRNOTAVAIL:
                 raise argparse.ArgumentTypeError('Cannot bind to specified IP address: "%s"' % addr)
         finally:
-            s.close()
+            try:
+                s.close()
+            except:
+                pass
         return addr
 
     @classmethod
@@ -1237,9 +1270,14 @@ class ArgHelper:
 
     def populate_recursive_servers(self):
         if not self.args.authoritative_analysis and not self.args.recursive_servers:
+            try:
+                resolver = Resolver.from_file(RESOLV_CONF, StandardRecursiveQueryCD, transport_manager=tm)
+            except ResolvConfError:
+                raise ResolvConfError('If servers are not specified with the %s option, then %s must have valid nameserver entries.' % \
+                        (self._arg_mapping['recursive_servers'], RESOLV_CONF))
             if (WILDCARD_EXPLICIT_DELEGATION, dns.rdatatype.NS) not in self.explicit_delegations:
                 self.explicit_delegations[(WILDCARD_EXPLICIT_DELEGATION, dns.rdatatype.NS)] = dns.rrset.RRset(WILDCARD_EXPLICIT_DELEGATION, dns.rdataclass.IN, dns.rdatatype.NS)
-            for i, server in enumerate(self._resolver._servers):
+            for i, server in enumerate(resolver._servers):
                 if IPAddr(server).version == 6:
                     rdtype = dns.rdatatype.AAAA
                 else:
@@ -1333,8 +1371,8 @@ class ArgHelper:
 
         if self.args.client_subnet:
             CustomQueryMixin.edns_options.append(self.args.client_subnet)
-        if self.args.nsid:
-            CustomQueryMixin.edns_options.append(self.args.nsid)
+        if not self.args.no_nsid:
+            CustomQueryMixin.edns_options.append(self.nsid_option())
         if self.args.cookie:
             CustomQueryMixin.edns_options.append(self.args.cookie)
 
@@ -1447,24 +1485,16 @@ class ArgHelper:
             zone.serve()
 
 def build_helper(logger, cmd, subcmd):
-    try:
-        resolver = Resolver.from_file(RESOLV_CONF, StandardRecursiveQueryCD, transport_manager=tm)
-    except ResolvConfError:
-        sys.stderr.write('File %s not found or contains no nameserver entries.\n' % RESOLV_CONF)
-        sys.exit(1)
-
-    arghelper = ArgHelper(resolver, logger)
+    arghelper = ArgHelper(logger)
     arghelper.build_parser('%s %s' % (cmd, subcmd))
     return arghelper
 
 def main(argv):
-    global tm
     global th_factories
     global explicit_delegations
     global odd_ports
 
     try:
-        _init_tm()
         arghelper = build_helper(logger, sys.argv[0], argv[0])
         arghelper.parse_args(argv[1:])
         logger.setLevel(arghelper.get_log_level())
@@ -1481,6 +1511,11 @@ def main(argv):
             arghelper.serve_zones()
         except argparse.ArgumentTypeError as e:
             arghelper.parser.error(str(e))
+        except ResolvConfError as e:
+            arghelper.parser.print_usage(sys.stderr)
+            sys.stderr.write("%(prog)s: error: %(errmsg)s\n" % {
+                             'prog': arghelper.parser.prog, 'errmsg': str(e) })
+            sys.exit(5)
         except (ZoneFileServiceError, MissingExecutablesError) as e:
             s = str(e)
             if s:
@@ -1513,6 +1548,10 @@ def main(argv):
             kwargs = {}
         dnsviz_meta = { 'version': DNS_RAW_VERSION, 'names': [lb2s(n.to_text()) for n in arghelper.names] }
 
+        NameServerMappingsForDomain.cleanup_resolvers()
+
+        _init_tm()
+
         name_objs = []
         if arghelper.args.input_file:
             cache = {}
@@ -1533,6 +1572,8 @@ def main(argv):
 
             name_objs = a.analyze(arghelper.names)
 
+        _cleanup_tm()
+
         name_objs = [x for x in name_objs if x is not None]
 
         if not name_objs:
@@ -1553,10 +1594,5 @@ def main(argv):
         logger.error('Interrupted.')
         sys.exit(4)
 
-    # tm is global (because of possible multiprocessing), so we need to
-    # explicitly close it here
-    finally:
-        _cleanup_tm()
-
 if __name__ == "__main__":
     main(sys.argv)
