@@ -9,7 +9,7 @@
 #
 # Copyright 2014-2016 VeriSign, Inc.
 #
-# Copyright 2016-2021 Casey Deccio
+# Copyright 2016-2024 Casey Deccio
 #
 # DNSViz is free software; you can redistribute it and/or modify
 # it under the terms of the GNU General Public License as published by
@@ -221,10 +221,19 @@ class DNSResponse:
         except IndexError:
             return None
 
-        try:
-            nsid_val = nsid_opt.data.decode('ascii')
-        except UnicodeDecodeError:
-            nsid_val = '0x' + lb2s(binascii.hexlify(nsid_opt.data))
+        # Backwards compatibility with dnspython < 2.6
+        if hasattr(dns.edns, 'NSIDOption'):
+            # dnspython >= 2.6 with NSIDOption
+            data = nsid_opt.nsid
+        else:
+            # dnspython < 2.6 with Generic Option
+            data = nsid_opt.data
+
+        if all(c >= 0x20 and c <= 0x7e for c in data):
+            nsid_val = data.decode('ascii')
+        else:
+            nsid_val = '0x' + lb2s(binascii.hexlify(data))
+
         return nsid_val
 
     def request_cookie_tag(self):
@@ -981,7 +990,10 @@ class DNSKEYMeta(DNSResponseComponent):
             d['servers'] = servers
 
             if map_ip_to_ns_name is not None:
-                ns_names = list(set([lb2s(map_ip_to_ns_name(s)[0][0].canonicalize().to_text()) for s in servers]))
+                try:
+                    ns_names = list(set([lb2s(map_ip_to_ns_name(s)[0][0].canonicalize().to_text()) for s in servers]))
+                except IndexError:
+                    ns_names = []
                 ns_names.sort()
                 d['ns_names'] = ns_names
 
@@ -1055,13 +1067,18 @@ class RRsetInfo(DNSResponseComponent):
     def __hash__(self):
         return hash(id(self))
 
+    @classmethod
+    def sorted_rdata_list(cls, rrset):
+        rdata_list = [RdataWrapper(x) for x in rrset]
+        rdata_list.sort()
+        return rdata_list
+
     @classmethod
     def rrset_canonicalized_to_wire(cls, rrset, name, ttl):
         s = b''
         name_wire = name.to_wire()
 
-        rdata_list = [RdataWrapper(x) for x in rrset]
-        rdata_list.sort()
+        rdata_list = cls.sorted_rdata_list(rrset)
 
         for rdataw in rdata_list:
             rdata = rdataw._rdata
@@ -1168,7 +1185,10 @@ class RRsetInfo(DNSResponseComponent):
             d['servers'] = servers
 
             if map_ip_to_ns_name is not None:
-                ns_names = list(set([lb2s(map_ip_to_ns_name(s)[0][0].canonicalize().to_text()) for s in servers]))
+                try:
+                    ns_names = list(set([lb2s(map_ip_to_ns_name(s)[0][0].canonicalize().to_text()) for s in servers]))
+                except IndexError:
+                    ns_names = []
                 ns_names.sort()
                 d['ns_names'] = ns_names
 
@@ -1533,3 +1553,17 @@ class TruncatedResponse(DNSResponseComponent):
 
     def __hash__(self):
         return hash(id(self))
+
+def dnskey_rrset_to_ds_rrset(dnskey_rrset_info, digest_alg_map, default_algs):
+    rrset = dns.rrset.RRset(dnskey_rrset_info.rrset.name, dnskey_rrset_info.rrset.rdclass, dns.rdatatype.DS)
+    for dnskey_rdata in dnskey_rrset_info.rrset:
+        dnskey_meta = DNSKEYMeta(dnskey_rrset_info.rrset.name, dnskey_rdata, dnskey_rrset_info.rrset.ttl)
+        dnskey_meta.servers_clients = copy.deepcopy(dnskey_rrset_info.servers_clients)
+        dnskey_msg = dnskey_meta.message_for_ds()
+        for digest_alg in digest_alg_map.get((dnskey_meta.rdata.algorithm, dnskey_meta.key_tag), default_algs):
+            digest = crypto.get_ds_digest(digest_alg, dnskey_meta.message_for_ds())
+            rdata = dns.rdtypes.ANY.DS.DS(dns.rdataclass.IN, dns.rdatatype.DS, dnskey_meta.key_tag, dnskey_meta.rdata.algorithm, digest_alg, digest)
+            rrset.add(rdata)
+    ds_rrset_info = RRsetInfo(rrset, dnskey_rrset_info.ttl_cmp)
+    ds_rrset_info.servers_clients = copy.deepcopy(dnskey_rrset_info.servers_clients)
+    return ds_rrset_info
