@@ -5,7 +5,7 @@
 #
 # Copyright 2014-2016 VeriSign, Inc.
 #
-# Copyright 2016-2021 Casey Deccio
+# Copyright 2016-2024 Casey Deccio
 #
 # DNSViz is free software; you can redistribute it and/or modify
 # it under the terms of the GNU General Public License as published by
@@ -73,8 +73,8 @@ MAX_PORT_BIND_ATTEMPTS=10
 MAX_WAIT_FOR_REQUEST=30
 HTTP_HEADER_END_RE = re.compile(r'(\r\n\r\n|\n\n|\r\r)')
 HTTP_STATUS_RE = re.compile(r'^HTTP/\S+ (?P<status>\d+) ')
-CONTENT_LENGTH_RE = re.compile(r'^Content-Length: (?P<length>\d+)', re.MULTILINE)
-CHUNKED_ENCODING_RE = re.compile(r'^Transfer-Encoding: chunked(\r\n|\r|\n)', re.MULTILINE)
+CONTENT_LENGTH_RE = re.compile(r'^Content-[Ll]ength: (?P<length>\d+)', re.MULTILINE)
+CHUNKED_ENCODING_RE = re.compile(r'^Transfer-[Ee]ncoding: chunked(\r\n|\r|\n)', re.MULTILINE)
 CHUNK_SIZE_RE = re.compile(r'^(?P<length>[0-9a-fA-F]+)(;[^\r\n]+)?(\r\n|\r|\n)')
 CRLF_START_RE = re.compile(r'^(\r\n|\n|\r)')
 
@@ -438,7 +438,7 @@ class DNSQueryTransportHandler(object):
             self.timeout = self.timeout_baseline
 
         if self._sock is not None:
-            # if a pre-existing socket is available for re-use, then use that
+            # if a preexisting socket is available for re-use, then use that
             # instead
             try:
                 self._reuse_socket()
@@ -798,7 +798,7 @@ class DNSQueryTransportHandlerHTTP(DNSQueryTransportHandlerMulti):
         self.sock = new_sock
 
     def _post_data(self):
-        return 'content=' + urlquote.quote(json.dumps(self.serialize_requests()))
+        return json.dumps(self.serialize_requests())
 
     def _authentication_header(self):
         if not self.username:
@@ -812,7 +812,7 @@ class DNSQueryTransportHandlerHTTP(DNSQueryTransportHandlerMulti):
 
     def init_req(self):
         data = self._post_data()
-        self.msg_send = codecs.encode('POST %s HTTP/1.1\r\nHost: %s\r\nUser-Agent: DNSViz/0.9.4\r\nAccept: application/json\r\n%sContent-Length: %d\r\nContent-Type: application/x-www-form-urlencoded\r\n\r\n%s' % (self.path, self.host, self._authentication_header(), len(data), data), 'latin1')
+        self.msg_send = codecs.encode('POST %s HTTP/1.1\r\nHost: %s\r\nUser-Agent: DNSViz/0.10.0\r\nAccept: application/json\r\n%sContent-Length: %d\r\nContent-Type: application/json\r\n\r\n%s' % (self.path, self.host, self._authentication_header(), len(data), data), 'latin1')
         self.msg_send_len = len(self.msg_send)
         self.msg_send_index = 0
 
@@ -1403,7 +1403,14 @@ class _DNSQueryTransportManager:
         self._event_map = {}
 
         self._close = threading.Event()
-        t = threading.Thread(target=self._loop)
+        # python3/python2 dual compatibility
+        try:
+            # python 3
+            t = threading.Thread(target=self._loop, daemon=True)
+        except TypeError:
+            # python 2
+            t = threading.Thread(target=self._loop)
+            t.daemon = True
         t.start()
 
     def close(self):
